using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Contents;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ContentRepository : IContentRepository
    {
        #region Private

        private const string RepositoryName = "ContentRepository";
        private const string RepositoryContentMenuItemName = "ContentMenuItemRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetContentIDFromUrlNameSignature = "GetContentIDFromUrlName";
        private const string GetIDByNameSignature = "GetIDByName";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetAllSignature = "GetAll";
        private const string SearchContentSignature = "SearchContent";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string AdvancedSearchContentSignature = "AdvancedSearchContent";
        private DataCache _dataCache = new DataCache( RepositoryName );
        private DataCache _dataCache_ContentMenuItem = new DataCache( RepositoryContentMenuItemName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "ContentLocale", "Content" } );

        private ContentRepositoryNoCache _repository = new ContentRepositoryNoCache();

        #endregion

        #region Constructors

        public ContentRepository()
        {
        }

        public ContentRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Content Save( Content content )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Content newContent = _repository.Save( content );
            _dataCache.ClearRepositoryCache();
            _dataCache_ContentMenuItem.ClearRepositoryCache();
            return newContent;
        }

        public Content GetOne( Culture culture, string contentID )
        {
            Content result = (Content) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, contentID );

            if (result == null)
            {
                result = _repository.GetOne( culture, contentID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, contentID );
            }

            return result;
        }

        public string GetContentIDFromUrlName( string urlName )
        {
            object contentID = _dataCache.GetValue( GetContentIDFromUrlNameSignature, _cacheDependency, urlName );

            if (contentID == null)
            {
                contentID = _repository.GetContentIDFromUrlName( urlName );
                _dataCache.SetValue( GetContentIDFromUrlNameSignature, contentID, urlName );
            }

            return contentID.ToString();
        }

        public string GetIDByName( Culture culture, string contentName )
        {
            object result = _dataCache.GetValue( GetIDByNameSignature, _cacheDependency, culture.CultureID, contentName );

            if (result == null)
            {
                result = _repository.GetIDByName( culture, contentName );
                _dataCache.SetValue( GetIDByNameSignature, result, culture.CultureID, contentName );
            }

            return result.ToString();
        }

        public void Delete( string contentID )
        {
            _repository.Delete( contentID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ILocale> GetLocales( string contentID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, contentID );

            if (list == null)
            {
                list = _repository.GetLocales( contentID );
                _dataCache.SetValue( GetLocalesSignature, list, contentID );
            }

            return list;
        }

        public IList<Content> GetAll( Culture culture, BoolFilter showInSiteMap, string sortBy )
        {
            IList<Content> list = (IList<Content>) _dataCache.GetValue( GetAllSignature, _cacheDependency, culture.CultureID, showInSiteMap.ToString(), sortBy );

            if (list == null)
            {
                list = _repository.GetAll( culture, showInSiteMap, sortBy );
                _dataCache.SetValue( GetAllSignature, list, culture.CultureID, showInSiteMap.ToString(), sortBy );
            }

            return list;
        }

        public IList<Content> SearchContent(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchContentSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchContent(
                    culture, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchContentSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );

            return (IList<Content>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        public IList<Content> AdvancedSearch(
           Culture culture,
           string storeID,
           string contentMenuItemIDs,
           string sortBy,
           string searchTerm,
           string[] searchFieldList,
           int startIndex,
           int endIndex,
           string searchType,
           out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                AdvancedSearchContentSignature,
                _cacheDependency,
                culture.CultureID,
                storeID,
                contentMenuItemIDs,
                sortBy,
                searchTerm,
                string.Join( ",", searchFieldList ),
                startIndex,
                endIndex,
                searchType
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.AdvancedSearch(
                    culture, storeID, contentMenuItemIDs, sortBy, searchTerm, searchFieldList, startIndex, endIndex, searchType, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchContentSignature,
                    result,
                    culture.CultureID,
                    storeID,
                    contentMenuItemIDs,
                    sortBy,
                    string.Join( ",", searchFieldList ),
                    startIndex,
                    endIndex,
                    searchType );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Content>) result.First;
        }
        #endregion
    }
}
