using System.Collections.Generic;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Base.Domain;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class CustomerRepository : ICustomerRepository
    {
        #region Private

        private const string RepositoryName = "CustomerRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetIDFromUserNameSignature = "GetIDFromUserName";
        private const string SearchCustomerSignature = "SearchCustomer";
        private const string ExportCustomerSignature = "ExportCustomer";
        private const string GetCustomerByEmailSignature = "GetCustomerByEmail";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetAllSignature = "GetAll";
        private const string GetLastestCustomersSignature = "GetLastestCustomers";
        private const string GetOneByFBUserIDSignature = "GetOneByFBUserID";

        private const string GetAllShippingAddressSignature = "GetAllShippingAddress";
        private const string GetShippingAddressByCustomerIDSignature = "GetShippingAddressByCustomerID";
        private const string GetShippingAddressTableSchemasSignature = "GetShippingAddressTableSchemas";
        private const string SearchShippingAddressSignature = "SearchShippingAddress";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "Customer", "ShippingAddress" } );

        private CustomerRepositoryNoCache _repository = new CustomerRepositoryNoCache();

        #endregion

        #region Constructors

        public CustomerRepository()
        {
        }

        public CustomerRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Customer Save( Customer customer )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Customer newCustomer = _repository.Save( customer );
            _dataCache.ClearRepositoryCache();
            return newCustomer;
        }

        public Customer GetOne( string customerID )
        {
            Customer result = (Customer) _dataCache.GetValue( GetOneSignature, _cacheDependency, customerID );

            if (result == null)
            {
                result = _repository.GetOne( customerID );
                _dataCache.SetValue( GetOneSignature, result, customerID );
            }

            return result;
        }

        public string GetIDFromUserName( string userName )
        {
            string result = (string) _dataCache.GetValue( GetIDFromUserNameSignature, _cacheDependency, userName );

            if (result == null || result == "0")
            {
                result = _repository.GetIDFromUserName( userName );
                _dataCache.SetValue( GetIDFromUserNameSignature, result, userName );
            }

            return result;
        }

        public void Delete( string customerID )
        {
            _repository.Delete( customerID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<Customer> SearchCustomer(
            string sortBy,
            SearchFilter searchFilter,
            BoolFilter isWholesale,
            int startIndex,
            int endIndex,
            out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchCustomerSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                isWholesale.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchCustomer(
                    sortBy, searchFilter, isWholesale, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchCustomerSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    isWholesale.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Customer>) result.First;
        }

        public IList<ShippingAddress> SearchShippingAddress(
            string customerID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchShippingAddressSignature,
                _cacheDependency,
                customerID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchShippingAddress(
                    customerID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );

                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchShippingAddressSignature,
                    result,
                    customerID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<ShippingAddress>) result.First;
        }

        public IList<Customer> ExportCustomer(
            string sortBy,
            SearchFilter searchFilter,
            BoolFilter isWholesale )
        {
            IList<Customer> list = (IList<Customer>) _dataCache.GetValue(
                ExportCustomerSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                isWholesale );

            if (list == null)
            {
                list = _repository.ExportCustomer( sortBy, searchFilter, isWholesale );
                _dataCache.SetValue(
                    ExportCustomerSignature,
                    sortBy,
                    searchFilter.ToString(),
                    isWholesale );
            }

            return list;
        }

        public IList<Customer> GetCustomerByEmail( string email )
        {
            IList<Customer> list = (IList<Customer>) _dataCache.GetValue(
                    GetCustomerByEmailSignature,
                    _cacheDependency,
                    email
                    );

            if (list == null)
            {
                list = _repository.GetCustomerByEmail( email );
                _dataCache.SetValue(
                    GetCustomerByEmailSignature,
                    list,
                    email
                    );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                    GetTableSchemaSignature, _cacheDependency
                    );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public IList<TableSchemaItem> GetShippingAddressTableSchemas()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                GetShippingAddressTableSchemasSignature, _cacheDependency
            );
            if (list == null)
            {
                list = _repository.GetShippingAddressTableSchemas();
                _dataCache.SetValue(
                    GetShippingAddressTableSchemasSignature,
                    list );
            }

            return list;
        }


        public void UpdateWholesalesLevel( int wholesaleLevel )
        {
            _repository.UpdateWholesalesLevel( wholesaleLevel );
            _dataCache.ClearRepositoryCache();
        }

        public IList<Customer> GetAll( string sortBy )
        {
            IList<Customer> list = (IList<Customer>) _dataCache.GetValue( GetAllSignature, _cacheDependency, sortBy );

            if (list == null)
            {
                list = _repository.GetAll( sortBy );

                _dataCache.SetValue( GetAllSignature, list, sortBy );
            }
            return list;
        }

        public IList<Customer> GetLastestCustomers( int limitCount )
        {
            IList<Customer> list = (IList<Customer>) _dataCache.GetValue( GetLastestCustomersSignature, _cacheDependency, limitCount );

            if (list == null)
            {
                list = _repository.GetLastestCustomers( limitCount );

                _dataCache.SetValue( GetLastestCustomersSignature, list, limitCount );
            }
            return list;
        }

        public IList<Customer> GetByIsQBExported( bool isQBExported, string existList )
        {
            IList<Customer> list = _repository.GetByIsQBExported( isQBExported, existList );
            return list;
        }

        public void SetDefaultQBData( string qbListID )
        {
            _repository.SetDefaultQBData( qbListID );
            _dataCache.ClearRepositoryCache();
        }

        public Customer GetOneByFBUserID( string FBUserID )
        {
            Customer result = (Customer) _dataCache.GetValue( GetOneByFBUserIDSignature, _cacheDependency, FBUserID );

            if (result == null)
            {
                result = _repository.GetOneByFBUserID( FBUserID );
                _dataCache.SetValue( GetOneByFBUserIDSignature, result, FBUserID );
            }

            return result;
        }

        public IList<ShippingAddress> GetShippingAddressByCustomerID( string customerID )
        {
            IList<ShippingAddress> list =
               (IList<ShippingAddress>) _dataCache.GetValue(
               GetShippingAddressByCustomerIDSignature,
               _cacheDependency,
               customerID );

            if (list == null)
            {
                list = _repository.GetShippingAddressByCustomerID( customerID );
                _dataCache.SetValue( GetShippingAddressByCustomerIDSignature, list, customerID );
            }

            return list;
        }

        public void ClearCustomerCache()
        {
            _dataCache.ClearRepositoryCache();
        }

        public IList<ShippingAddress> GetAllShippingAddress( string sortBy )
        {
            IList<ShippingAddress> list = (IList<ShippingAddress>) _dataCache.GetValue(
                GetAllShippingAddressSignature,
                _cacheDependency,
                sortBy );

            if (list == null)
            {
                list = _repository.GetAllShippingAddress( sortBy );

                _dataCache.SetValue( GetAllShippingAddressSignature, list, sortBy );
            }
            return list;
        }

        public void DeleteAllShippingAddress()
        {
            _repository.DeleteAllShippingAddress();
            _dataCache.ClearRepositoryCache();
        }

        #endregion
    }
}
