using System;
using System.Collections.Generic;
using System.Text;

using Vevo.Data.Implementation;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using System.Collections;
using System.Data;
using Vevo.Base.Data;
using Vevo.Base.Domain;


namespace Vevo.Data
{
    public class ShippingZoneGroupRepositoryNoCache : IShippingZoneGroupRepository
    {
        #region Private

        private ShippingZoneItemAccess _shippingZoneItemAccess = new ShippingZoneItemAccess();

        private const string AllShippingZoneGroupFields = "ZoneGroupID, ZoneName ";

        private ShippingZoneGroup CreateOneFromRow( DataRow row )
        {
            ShippingZoneGroup zoneGroup = new ShippingZoneGroup();
            zoneGroup.ZoneGroupID = row["ZoneGroupID"].ToString();
            zoneGroup.ZoneName = row["ZoneName"].ToString();

            return zoneGroup;
        }

        private IList<ShippingZoneGroup> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<ShippingZoneGroup> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<ShippingZoneGroup> list = new List<ShippingZoneGroup>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        private ShippingZoneGroup Create( ShippingZoneGroup zoneGroup )
        {
            zoneGroup.ZoneGroupID = DataAccess.ExecuteScalar(
                "INSERT INTO ShippingZoneGroup ( ZoneName ) " +
                "VALUES ( @ZoneName );" +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( zoneGroup.ZoneName ) );

            return zoneGroup;
        }

        private ShippingZoneGroup Update( ShippingZoneGroup zoneGroup )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE ShippingZoneGroup " +
                "SET ZoneName = @ZoneName " +
                "WHERE ZoneGroupID  = @ZoneGroupID;",
                DataAccess.CreateParameterString( zoneGroup.ZoneName ),
                DataAccess.CreateParameterString( zoneGroup.ZoneGroupID ) );

            return zoneGroup;
        }


        #endregion

        #region IShippingZoneGroupRepository Members

        public IList<ShippingZoneItem> GetShippingZoneItemByZoneGroupID( string zoneGroupID )
        {
            return _shippingZoneItemAccess.GetShippingZoneItemByZoneGroupID( zoneGroupID );
        }

        public IList<ShippingZoneItem> SearchShippingZoneItems(
            string zoneGroupID, string sortBy, SearchFilter searchFilter, int startIndex, int endIndex, out int howManyItems )
        {
            return _shippingZoneItemAccess.SearchShippingZoneItems(
                zoneGroupID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
        }

        public IList<TableSchemaItem> GetShippingZoneItemTableSchemas()
        {
            return _shippingZoneItemAccess.GetShippingZoneItemTableSchemas();
        }

        public ShippingZoneGroup Save( ShippingZoneGroup zoneGroup )
        {
            ShippingZoneGroup result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (ConvertUtilities.ToInt32( zoneGroup.ZoneGroupID ) == 0)
                    result = Create( zoneGroup );
                else
                    result = Update( zoneGroup );

                _shippingZoneItemAccess.SaveAll(
                    zoneGroup.ZoneGroupID,
                    zoneGroup.ZoneItem );

                transaction.Complete();
            }

            return result;

        }

        public ShippingZoneGroup GetOne( string zoneGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
               "SELECT " + AllShippingZoneGroupFields +
               "FROM ShippingZoneGroup " +
               "WHERE ZoneGroupID  = @ZoneGroupID; ",
               DataAccess.CreateParameterString( zoneGroupID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return ShippingZoneGroup.Null;
        }

        public void Delete( string zoneGroupID )
        {
            ShippingZoneGroup zoneGroup = GetOne( zoneGroupID );
            zoneGroup.ZoneItem.Clear();

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                   "DELETE " +
                   "FROM ShippingZoneGroup " +
                   "WHERE ZoneGroupID  = @ZoneGroupID; ",
                   DataAccess.CreateParameterString( zoneGroupID ) );

                _shippingZoneItemAccess.SaveAll( zoneGroup.ZoneGroupID, zoneGroup.ZoneItem );

                transaction.Complete();
            }
        }

        public IList<ShippingZoneGroup> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingZoneGroupFields +
                "FROM ShippingZoneGroup " +
                "ORDER BY " + sortBy );

            return ConvertTableToList( table );
        }

        public IList<ShippingZoneGroup> SearchShippingZoneGroups(
            string sortBy,
            SearchFilter searchFilter, 
            int startIndex, 
            int endIndex, 
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingZoneGroupFields +
                "FROM ShippingZoneGroup " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
            );

            howManyItems = table.Rows.Count;

            DataColumn[] keys = new DataColumn[] { table.Columns["ZoneGroupID"] };
            table.PrimaryKey = keys;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchemas()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllShippingZoneGroupFields +
                "FROM ShippingZoneGroup " +
                "WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
