using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Tax;
using Vevo.Domain.DataInterfaces;
using Vevo.Data.Implementation;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class TaxClassRepositoryNoCache : ITaxClassRepository
    {
        #region Private

        private TaxClassRuleAccess _taxClassRuleAccess = new TaxClassRuleAccess();
        private TaxClassAccess _taxClassAccess = new TaxClassAccess();

        private const string AllTaxClassFields = "TaxClassID, TaxClassName ";

        private TaxClass CreateOneTaxClassFromRow( DataRow row )
        {
            TaxClass taxClass = new TaxClass();
            taxClass.TaxClassID = row["TaxClassID"].ToString();
            taxClass.TaxClassName = row["TaxClassName"].ToString();

            return taxClass;
        }

        private IList<TaxClass> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<TaxClass> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<TaxClass> list = new List<TaxClass>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneTaxClassFromRow( table.Rows[i] ) );

            return list;
        }        

        #endregion

        // Functions that are intended to be used internally inside aggregates.
        #region Aggregate Methods

        public IList<TaxClassRule> GetTaxClassRulesByTaxClassID( string taxClassID )
        {
            return _taxClassRuleAccess.GetByTaxClassID( taxClassID );
        }

        public TaxClassRule GetOneTaxClassRule( string taxClassRuleID )
        {
            return _taxClassRuleAccess.GetOne( taxClassRuleID );
        }

        public IList<TaxClassRule> GetAllTaxClassRules( string sortBy )
        {
            return _taxClassRuleAccess.GetAll( sortBy );
        }

        public IList<TaxClassRule> SearchTaxClassRules(
            string taxClassID,
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            return _taxClassRuleAccess.SearchTaxClassRules( taxClassID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
        }

        public IList<TableSchemaItem> GetTaxClassRuleTableSchemas()
        {
            return _taxClassRuleAccess.GetTableSchemas();
        }

        #endregion

        #region Public Methods

        public TaxClass Save( TaxClass taxClassGroup )
        {
            TaxClass result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (ConvertUtilities.ToInt32( taxClassGroup.TaxClassID ) == 0)
                    result = _taxClassAccess.Create( taxClassGroup );
                else
                    result = _taxClassAccess.Update( taxClassGroup );

                _taxClassRuleAccess.SaveAll(
                    taxClassGroup.TaxClassID,
                    taxClassGroup.TaxClassRule );

                transaction.Complete();
            }
            return result;
        }

        public TaxClass GetOne( string taxClassID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllTaxClassFields +
                "FROM TaxClass " +
                "WHERE TaxClassID = @TaxClassID; ",
                DataAccess.CreateParameterString( taxClassID ) );

            if (table.Rows.Count > 0)
                return CreateOneTaxClassFromRow( table.Rows[0] );
            else
                return TaxClass.Null;
        }

        public void Delete( string taxClassID )
        {
            TaxClass taxClass = GetOne( taxClassID );
            taxClass.TaxClassRule.Clear();

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                _taxClassAccess.Delete( taxClassID );

                _taxClassRuleAccess.SaveAll(
                    taxClass.TaxClassID,
                    taxClass.TaxClassRule );

                transaction.Complete();
            }
        }

        public IList<TaxClass> GetAll( string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllTaxClassFields +
                "FROM TaxClass " +
                "ORDER BY " + sortBy );

            return ConvertTableToList( table );
        }

        public IList<TaxClass> SearchTaxClasses(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            DataTable table = _taxClassAccess.SearchTaxClasses(
                sortBy,
                searchFilter,
                out howManyItems );

            return ConvertTableToList( table, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchemas()
        {
            return _taxClassAccess.GetTableSchemas();
        }
        #endregion
    }
}
