using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.IO;
using System.Web;
using System.Xml;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Shipping;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;
using Vevo.Shared.SystemServices;
using Vevo.WebAppLib;
using Vevo.Domain.Discounts;
using Vevo.Base.Domain;

namespace Vevo.Domain.Shipping.Ups
{
    /// <summary>
    /// Summary description for UpsShippingGateway
    /// </summary>
    public class UpsShippingGateway : IShippingGateway
    {
        #region Private

        private const string ServiceListCacheName = "UpsShippingGateway_ServiceList";

        private ShippingOption _shippingOption;


        private bool IsTest
        {
            get
            {
                if (DataAccessContext.Configurations.GetBoolValue( "RTShippingUpsTestMode" ))
                    return true;
                else
                    return false;
            }
        }

        private bool VerifyResponse( XmlDocument source )
        {

            XmlNodeList list = source.GetElementsByTagName( "ResponseStatusCode" );

            if (list[0].InnerText == "1")
                return true;
            else
                return false;
        }

        private string GetWeightUnit()
        {
            if (DataAccessContext.Configurations.GetValue( "WeightUnit" ) == "KG")
                return "KGS";
            else
                return "LBS";
        }

        private double GetWeightInPounds( double weight )
        {
            if (DataAccessContext.Configurations.GetValue( "WeightUnit" ) == "KG")
            {
                return WeightUtilities.ConvertKgToPounds( weight );
            }
            else
                return weight;
        }

        private double GetOrderWeight( double currentWeight )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "RTShippingUpsUseMinOrderWeight" ))
            {
                double minimumWeight =
                    DataAccessContext.Configurations.GetDoubleValue( "RTShippingUpsMinOrderWeight" );
                if (currentWeight < minimumWeight)
                    return GetWeightInPounds( minimumWeight );
            }
            return GetWeightInPounds( currentWeight );
        }

        private string GetResidentialElementText( bool residential )
        {
            if (residential)
                return "<ResidentialAddressIndicator />";
            else
                return String.Empty;
        }

        private string GenerateData( ShippingAddress shippingAddress, double weight )
        {
            return String.Format(
            "<?xml version=\"1.0\"?>" +
            "<AccessRequest xml:lang=\"en-US\">" +
            "   <AccessLicenseNumber>{0}</AccessLicenseNumber>" +
            "   <UserId>{1}</UserId>" +
            "   <Password>{2}</Password>" +
            "</AccessRequest>" +
            "<?xml version=\"1.0\"?>" +
            "<RatingServiceSelectionRequest xml:lang=\"en-US\">" +
            "   <Request>" +
            "       <TransactionReference>" +
            "           <CustomerContext>Rating and Service</CustomerContext>" +
            "           <XpciVersion>1.0001</XpciVersion>" +
            "       </TransactionReference>" +
            "       <RequestAction>Rate</RequestAction>" +
            "       <RequestOption>shop</RequestOption>" +
            "   </Request>" +
            "   <PickupType>" +
            "       <Code>{3}</Code>" +
            "   </PickupType>" +
            "   <Shipment>" +
            "       <Shipper>" +
            "           <Address>" +
            "               <PostalCode>{4}</PostalCode>" +
            "               <CountryCode>{5}</CountryCode>" +
            "           </Address>" +
            "       </Shipper>" +
            "       <ShipTo>" +
            "           <Address>" +
            "               <PostalCode>{6}</PostalCode>" +
            "               <CountryCode>{7}</CountryCode>" +
            "               {8}" +
            "           </Address>" +
            "       </ShipTo>" +
            "       <Package>" +
            "           <PackagingType>" +
            "               <Code>{9}</Code>" +
            "               <Description>Package</Description>" +
            "           </PackagingType>" +
            "           <Description>Rate Shopping</Description>" +
            "           <PackageWeight>" +
            "               <UnitOfMeasurement>" +
            "                   <Code>{10}</Code>" +
            "               </UnitOfMeasurement>" +
            "               <Weight>{11}</Weight>" +
            "           </PackageWeight>" +
            "       </Package>" +
            "       <ShipmentServiceOptions/>" +
            "   </Shipment>" +
            "</RatingServiceSelectionRequest>",
            DataAccessContext.Configurations.GetValue( "RTShippingUpsAccessLicenseNumber" ),
            DataAccessContext.Configurations.GetValue( "RTShippingUpsUserID" ),
            DataAccessContext.Configurations.GetValue( "RTShippingUpsPassword" ),
            DataAccessContext.Configurations.GetValue( "RTShippingUpsPickupType" ),
            DataAccessContext.Configurations.GetValue( "RTShippingUpsMerchantZip" ),
            DataAccessContext.Configurations.GetValue( "RTShippingUpsMerchantCountry" ),
            shippingAddress.Zip,
            shippingAddress.Country,
            GetResidentialElementText( shippingAddress.Residential ),
            "00",
            GetWeightUnit(),
            GetOrderWeight( weight ) );
        }

        private XmlDocument GetData( ShippingAddress shippingAddress, double weight )
        {
            string urlPost = String.Empty;
            if (IsTest)
            {
                urlPost = "https://wwwcie.ups.com/ups.app/xml/Rate";
            }
            else
            {
                urlPost = "https://www.ups.com/ups.app/xml/Rate";
            }

            HttpService httpService = new HttpService();
            string responseData = httpService.PostData(
                    urlPost,
                    GenerateData( shippingAddress, weight ) );

            XmlDocument xmlData = new XmlDocument();

            xmlData.LoadXml( responseData );

            return xmlData;
        }

        private XmlNodeList GetChildTagByName( XmlNode source, string tagName )
        {
            return source[tagName].GetElementsByTagName( "*" );
        }

        private bool IsServiceSelected( string serviceCode )
        {
            string[] serviceList = DataAccessContext.Configurations.GetValueList( "RTShippingUpsService" );
            foreach (string code in serviceList)
            {
                if (code == serviceCode)
                    return true;
            }
            return false;
        }

        private bool IsServiceFreeShipping( string serviceCode, decimal subTotal )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "RTShippingUpsFreeShippingEnabled" )
                && subTotal > DataAccessContext.Configurations.GetDecimalValue( "RTShippingUpsFreeShippingCost" ))
            {
                string[] serviceList = DataAccessContext.Configurations.GetValueList(
                    "RTShippingUpsFreeShippingService" );
                foreach (string service in serviceList)
                {
                    if (service == serviceCode)
                        return true;
                }
            }
            return false;
        }

        private bool IsAllowedFreeShippingCostItem( string serviceCode, bool isContainFreeShippingItem )
        {
            if (isContainFreeShippingItem)
            {
                string[] serviceList = DataAccessContext.Configurations.GetValueList(
                    "RTShippingUpsServiceAllowedSetFree" );

                foreach (string service in serviceList)
                {
                    if (service == serviceCode)
                        return true;
                }
            }
            return false;
        }

        private bool IsAllowedUseCouponFreeShippingCost( string serviceCode, Coupon coupon )
        {
            if (coupon.DiscountType == Coupon.DiscountTypeEnum.FreeShipping)
            {
                string[] serviceList = DataAccessContext.Configurations.GetValueList(
                    "RTShippingUpsServiceAllowedUseCouponFree" );

                foreach (string service in serviceList)
                {
                    if (service == serviceCode)
                        return true;
                }

            }
            return false;
        }

        private string CreateDetailedError( XmlDocument xmlData )
        {
            string errorCode = xmlData.GetElementsByTagName( "ErrorCode" )[0].InnerText;
            return
                "Error (" + errorCode + "): "
                + xmlData.GetElementsByTagName( "ErrorDescription" )[0].InnerText;
        }

        private void PopulateShippingChoices(
            IList<ShippingChoice> shippingChoices,
            XmlDocument xmlData,
            decimal subtotal,
            bool isContainFreeShippingItem,
            Coupon coupon )
        {
            XmlNodeList serviceList = xmlData.GetElementsByTagName( "RatedShipment" );

            foreach (XmlNode node in serviceList)
            {
                XmlNodeList root = GetChildTagByName( node, "Service" );

                string serviceCode = root[0].InnerText;

                if (IsServiceSelected( serviceCode ))
                {
                    string name = GetServiceByServiceCode( serviceCode );
                    decimal shippingCost;

                    if (IsServiceFreeShipping( serviceCode, subtotal )
                        || IsAllowedFreeShippingCostItem( serviceCode, isContainFreeShippingItem )
                        || IsAllowedUseCouponFreeShippingCost( serviceCode, coupon )
                        )
                    {
                        shippingCost = 0;
                    }
                    else
                    {

                        //*** Check currency code for returned value
                        root = GetChildTagByName( node, "TotalCharges" );
                        shippingCost =
                            ConvertUtilities.ToDecimal( root[1].InnerText ) +
                            DataAccessContext.Configurations.GetDecimalValue( "RTShippingUpsMarkup" );

                    }

                    ShippingChoice option = new ShippingChoice(
                        name,
                        shippingCost,
                        _shippingOption.GetHandlingFee(
                            DataAccessContext.Configurations.GetBoolValue(
                                "HandlingFeeEnabled" ), shippingCost ) );

                    shippingChoices.Add( option );
                }
            }
        }

        #endregion


        #region Public Constants

        public const string UpsServiceListFilePath = "~/DataFiles/Shipping/UPSService.xml";

        #endregion


        #region Constructors

        public UpsShippingGateway( ShippingOption shippingOption )
        {
            _shippingOption = shippingOption;
        }

        #endregion


        #region Public Methods

        public static DataTable ReadServiceFile()
        {
            return ReadServiceFile( HttpContext.Current.Server.MapPath( UpsServiceListFilePath ) );
        }

        public static DataTable ReadServiceFile( string serviceListFullFilePath )
        {
            DataTable serviceList = (DataTable) HttpContext.Current.Cache[ServiceListCacheName];
            if (serviceList == null)
            {
                DataSet ds = new DataSet();
                if (File.Exists( serviceListFullFilePath ))
                    ds.ReadXml( serviceListFullFilePath );
                else
                    throw new Exception( "File Not Found : " + serviceListFullFilePath );

                serviceList = ds.Tables[0];

                HttpContext.Current.Cache[ServiceListCacheName] = serviceList;
            }

            return serviceList;
        }

        public static string GetServiceByServiceCode( string code )
        {
            string result = String.Empty;
            DataTable serviceList = ReadServiceFile();
            DataRow[] rows = serviceList.Select( "Code = '" + code + "'" );

            if (rows.Length != 0)
                result = rows[0]["Name"].ToString();
            return result;
        }

        #endregion


        #region IShippingGateway Members

        public IList<ShippingChoice> RequestShippingChoices(
            ShippingAddress shippingAddress,
            Orders.Cart shoppingCart,
            decimal subtotal,
            bool isContainFreeShippingItem,
            Coupon coupon,
            out string errorMessage,
            out string restriction )
        {
            errorMessage = String.Empty;
            restriction = String.Empty;

            IList<ShippingChoice> shippingChoices = new List<ShippingChoice>();

            try
            {
                XmlDocument xmlData = GetData( shippingAddress, shoppingCart.GetTotalWeight() );

                if (!VerifyResponse( xmlData ))
                {
                    errorMessage = CreateDetailedError( xmlData );
                }
                else
                {
                    PopulateShippingChoices( shippingChoices, xmlData, subtotal, isContainFreeShippingItem, coupon );
                }
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                if (!DataAccessContext.Configurations.GetBoolValue( "ShowDetailAjaxErrorMessage" ))
                    errorMessage = ex.Message;
                else
                    throw;
            }

            return shippingChoices;
        }

        #endregion
    }
}